#include "SnapshotSelectorWidget.h"

#include <QCheckBox>
#include <QComboBox>
#include <QHBoxLayout>
#include <QVBoxLayout>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::armem::gui
{
    client::query::SnapshotSelector& SnapshotSelectorWidget::selector()
    {
        return _selector;
    }

    query::data::QueryTarget SnapshotSelectorWidget::queryTarget() const
    {
        return _queryTarget;
    }


    SnapshotSelectorWidget::SnapshotSelectorWidget()
    {
        _pageLayout = new QVBoxLayout();
        setLayout(_pageLayout);

        connect(this, &This::queryOutdated, this, &This::updateSelector);
        connect(this, &This::queryTargetOutdated, this, &This::updateQueryTarget);

        {
            QHBoxLayout* typeLayout = new QHBoxLayout();

            // snapshot select box
            _queryComboBox = new QComboBox();
            _queryComboBox->addItems({"All", "Single", "Time Range", "Index Range"});
            _queryComboBox->setCurrentIndex(3);

            typeLayout->addWidget(_queryComboBox);

            connect(_queryComboBox, &QComboBox::currentTextChanged, this, &This::showSelectedFormForQuery);
            connect(_queryComboBox, &QComboBox::currentTextChanged, this, &This::queryOutdated);

            // query type select box
            _queryTargetComboBox = new QComboBox();
            _queryTargetComboBox->addItems({"WM", "WM & LTM", "LTM (debug only)"});
            _queryTargetComboBox->setCurrentIndex(0);

            typeLayout->addWidget(_queryTargetComboBox);

            connect(_queryTargetComboBox, &QComboBox::currentTextChanged, this, &This::queryTargetOutdated);

            _pageLayout->addLayout(typeLayout);
        }

        // Build and add the forms.
        _queryForms.clear();
        addForm("All", new SnapshotFormAll());
        addForm("Single", new SnapshotFormSingle());
        addForm("Time Range", new SnapshotFormTimeRange());
        addForm("Index Range", new SnapshotFormIndexRange());

        // Add query targets
        _queryTargets.insert({"WM", query::data::QueryTarget::WM});
        _queryTargets.insert({"WM & LTM", query::data::QueryTarget::WMAndLTM});
        _queryTargets.insert({"LTM (debug only)", query::data::QueryTarget::LTM});

        showSelectedFormForQuery(_queryComboBox->currentText());
        updateSelector();
        updateQueryTarget();
    }

    void SnapshotSelectorWidget::updateSelector()
    {
        this->_selector = _queryForms.at(_queryComboBox->currentText())->makeEntitySelector();
        emit queryChanged();
    }

    void SnapshotSelectorWidget::updateQueryTarget()
    {
        this->_queryTarget = _queryTargets.at(_queryTargetComboBox->currentText());
        emit queryTargetChanged();
    }

    void SnapshotSelectorWidget::showSelectedFormForQuery(QString selected)
    {
        for (auto& [name, form] : _queryForms)
        {
            form->setVisible(name == selected);
        }
    }

    void SnapshotSelectorWidget::addForm(const QString& key, SnapshotForm* form)
    {
        auto r = _queryForms.emplace(key, form);
        _pageLayout->addWidget(r.first->second);
        connect(r.first->second, &SnapshotForm::queryChanged, this, &This::updateSelector);
    }

    void SnapshotSelectorWidget::hideAllForms()
    {
        for (auto& [_, form] : _queryForms)
        {
            form->setVisible(false);
        }
    }


}
