/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include "../detail/ContainerSerializerBase.h"

// ArmarX
#include "../../../../../navigator/type/container/Dict.h"


namespace armarx::aron::cppserializer::serializer
{
    class DictSerializer;
    typedef std::shared_ptr<DictSerializer> DictSerializerPtr;

    class DictSerializer :
        public detail::ContainerSerializerBase<typenavigator::DictNavigator, DictSerializer>
    {
    public:
        using PointerType = DictSerializerPtr;

    public:
        // constructors
        DictSerializer() = delete;
        DictSerializer(const typenavigator::DictNavigatorPtr&);

        // virtual implementations
        virtual CppBlockPtr getWriteTypeBlock(const std::string&) const override;
        virtual CppBlockPtr getResetHardBlock(const std::string&) const override;
        virtual CppBlockPtr getResetSoftBlock(const std::string&) const override;
        virtual CppBlockPtr getWriteBlock(const std::string&) const override;
        virtual CppBlockPtr getReadBlock(const std::string&) const override;
        virtual CppBlockPtr getEqualsBlock(const std::string&, const std::string&) const override;

    private:
        // Members
        static constexpr const char* DICT_ITERATOR_ACCESSOR = "_iterator";
        static constexpr const char* DICT_KEY_ACCESSOR = "_key";
        static constexpr const char* DICT_VALUE_ACCESSOR = "_value";
    };
}
