/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Tuple.h"


namespace armarx::aron::typenavigator
{
    // constructors
    TupleNavigator::TupleNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eTuple, path)
    {
    }

    TupleNavigator::TupleNavigator(const type::AronTuplePtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eTuple, path),
        detail::NavigatorBase<type::AronTuple, TupleNavigator>(o)
    {
        for (const auto& t : this->aron->elementTypes)
        {
            acceptedTypes.push_back(FromAronType(t));
        }
    }

    // operators
    bool TupleNavigator::operator==(const TupleNavigator& other) const
    {
        if (getMaybe() != other.getMaybe())
        {
            return false;
        }

        unsigned int i = 0;
        for (const auto& nav : acceptedTypes)
        {
            if (nav != other.getAcceptedType(i++))
            {
                return false;
            }
        }
        return true;
    }

    type::AronTuplePtr TupleNavigator::toAronTuplePtr() const
    {
        if (acceptedTypes.empty())
        {
            throw error::AronException("TupleNavigator", "getCastedResult", "No accepted types set", getPath());
        }
        return this->aron;
    }

    // public member functions
    std::vector<NavigatorPtr> TupleNavigator::getAcceptedTypes() const
    {
        return acceptedTypes;
    }

    NavigatorPtr TupleNavigator::getAcceptedType(unsigned int i) const
    {
        return acceptedTypes[i];
    }

    void TupleNavigator::addAcceptedType(const NavigatorPtr& v)
    {
        ARMARX_CHECK_NOT_NULL(v);
        this->aron->elementTypes.push_back(v->toAronTypePtr());
        acceptedTypes.push_back(v);
    }

    // virtual implementations
    std::vector<NavigatorPtr> TupleNavigator::getChildren() const
    {
        return acceptedTypes;
    }

    size_t TupleNavigator::childrenSize() const
    {
        return acceptedTypes.size();
    }

    std::string TupleNavigator::getName() const
    {
        std::vector<std::string> names;
        for (const auto& n : acceptedTypes)
        {
            names.push_back(n->getName());
        }
        return "AronTupleType<" + simox::alg::to_string(names, ", ") + ">";
    }
}

