/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base class
#include "../detail/NDArrayNavigatorBase.h"

namespace armarx::aron::typenavigator
{
    class EigenMatrixNavigator;
    typedef std::shared_ptr<EigenMatrixNavigator> EigenMatrixNavigatorPtr;

    class EigenMatrixNavigator :
        virtual public detail::NDArrayNavigatorBase<type::AronEigenMatrix, EigenMatrixNavigator>
    {
    public:
        // constructors
        EigenMatrixNavigator(const Path& path = Path());
        EigenMatrixNavigator(const type::AronEigenMatrixPtr&, const Path& path = Path());

        // operators
        virtual bool operator==(const EigenMatrixNavigator&) const override;

        // public member functions
        std::string checkTypename(const std::string&) const;

        unsigned int getRows() const;
        unsigned int getCols() const;
        std::string getTypename() const;

        void setRows(int);
        void setCols(int);
        void setTypename(const std::string&);

        type::AronEigenMatrixPtr toAronEigenMatrixPtr() const;

        // virtual implementations
        virtual std::string getName() const override;

    public:
        const std::map<std::string, std::vector<std::string>> ACCEPTED_TYPES =
        {
            {"short", {}},
            {"int", {}},
            {"long", {}},
            {"float", {}},
            {"double", {}},
            //{"complex float", {"std::complex<float>"}},
            //{"complex double", {"std::complex<double>"}},
        };
    };
}
